"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const assert = require("assert");
const _ = require("lodash");
// import { Vars } from "../helpers";
const CRUDMiddleware_1 = require("./CRUDMiddleware");
/*************************************************************************
 *
 * Troven CONFIDENTIAL
 * __________________
 *
 *  (c) 2017-2020 Troven Ventures Pty Ltd. All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Troven Pty Ltd and its licensors,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Troven Pty Ltd
 * and its suppliers and may be covered by International and Regional Patents,
 * patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Troven Pty Ltd.
 */
class NestedTreeStore {
    constructor() {
        this.name = "datatree";
        this.store = null;
        this.id_field_name = "items";
        this.collection_field_name = "items";
        this.options = null;
        this.context = null;
    }
    getIDField() {
        return this.id_field_name;
    }
    getCollectionField() {
        return this.collection_field_name;
    }
    getStoreMiddleware() {
        return new CRUDMiddleware_1.CRUDMiddleware(this);
    }
    install(context, options) {
        assert(options.store, "missing options.store");
        this.context = context;
        this.options = options;
        this.id_field_name = options.id_field || this.id_field_name;
        this.collection_field_name = options.via || this.collection_field_name;
        let store_options = options[options.store] || false;
        let store = context.plugins.get(options.store);
        assert(this.store, "unknown options.store: " + options.store);
        assert(store_options, "unknown store.options: " + options.store);
        store.install(context, store_options);
        this.store = store; // force type coercion
        context.middleware.set(this.getStoreMiddleware());
        context.log({ code: "treestore:installed", database: this.options.database, id_field: this.id_field_name, collection_field: this.collection_field_name });
        return this;
    }
    redact(meta) {
        delete meta[this.collection_field_name];
        return meta;
    }
    nestedCallback(oper, meta, req_options, options, callback) {
        false && meta;
        false && req_options;
        assert(callback, "missing nested.callback");
        assert(_.isFunction(callback), "invalid.callback");
        // let self = this;
        return (err, items) => {
            false && err;
            // TODO: iterate 'read' for each item
            let item = items[0];
            // copy data's id into meta and req_options
            let nested_meta = _.extend({}, meta);
            nested_meta[this.id_field_name] = item[this.id_field_name];
            let nested_options = _.extend({}, req_options);
            nested_options[this.id_field_name] = item[this.id_field_name];
            // iterate over items
            this.read(oper, meta, nested_options, options, callback);
        };
    }
    healthy(req_options, callback) {
        return this.store.healthy(req_options, callback);
    }
    query(oper, meta, req_options, options, callback) {
        this.store.list(oper, meta, req_options, options, this.nestedCallback(oper, meta, req_options, options, callback));
    }
    create(oper, meta, req_options, options, callback) {
        meta = this.redact(meta);
        req_options = this.redact(req_options);
        // TODO: handle arrays as set of creates
        this.store.create(oper, meta, req_options, options, callback);
    }
    read(oper, meta, req_options, options, callback) {
        this.store.read(oper, meta, req_options, options, this.nestedCallback(oper, meta, req_options, options, callback));
    }
    list(oper, meta, req_options, options, callback) {
        this.store.list(oper, meta, req_options, options, this.nestedCallback(oper, meta, req_options, options, callback));
    }
    update(oper, meta, req_options, options, callback) {
        meta = this.redact(meta);
        req_options = this.redact(req_options);
        // TODO: handle arrays as set of creates
        this.store.update(oper, meta, req_options, options, callback);
    }
    delete(oper, meta, req_options, options, callback) {
        // TODO: handle arrays as set of creates
        this.store.delete(oper, meta, req_options, options, callback);
    }
}
exports.NestedTreeStore = NestedTreeStore;
//# sourceMappingURL=NestedTreeStore.js.map