"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const request = require("request");
const Vars_1 = require("./Vars");
const _ = require("lodash");
const http = require("http");
const https = require("https");
const url_1 = require("url");
class APIs {
    constructor() {
    }
    static get(_context, _url) {
        let url = new url_1.URL(_url);
        let client = url.protocol == "https:" ? https : http;
        return new Promise((resolve, reject) => {
            try {
                client.get(url, (res) => {
                    resolve(res);
                });
            }
            catch (err) {
                console.log("OOPS: %s", err);
                reject(err);
            }
        });
    }
    static request(context, url, _headers) {
        return new Promise((resolve, reject) => {
            // call backend API
            APIs.DEBUG && context.log({ code: "feature:helpers:request:api", url: url, headers: _headers });
            request({ uri: url, headers: _headers }, function (err, httpResponse, body) {
                if (err || !httpResponse) {
                    context.error({ code: "feature:helpers:api:failed", url: url, httpResponse: httpResponse ? true : false, error: err ? err.toString() : "no error" });
                    reject(err);
                }
                else {
                    APIs.DEBUG && context.log({ code: "feature:helpers:api:ok", url: url, body: body });
                    try {
                        resolve(JSON.parse(body));
                    }
                    catch (e) {
                        resolve(body);
                    }
                }
            });
        });
    }
    static named_request(context, api_name, url, _headers) {
        return new Promise((resolve, reject) => {
            // call backend API
            APIs.DEBUG && context.log({ code: "feature:render:api", api: api_name, url: url, headers: _headers });
            request({ uri: url, headers: _headers }, function (err, httpResponse, body) {
                if (err || !httpResponse) {
                    context.error({ code: "feature:render:api:failed", api: api_name, url: url, httpResponse: httpResponse ? true : false, error: err ? err.toString() : "no error" });
                    reject(err);
                }
                else {
                    try {
                        let result = {};
                        result[api_name] = JSON.parse(body);
                        resolve(result);
                    }
                    catch (e) {
                        reject(e);
                    }
                }
            });
        });
    }
    /**
     * Collect a set of API responses as promise
     *
     * @param apis
     * @param params
     * @param token
     */
    static requests(context, apis, params, token) {
        let _requests = [];
        // iterate over set of APIs
        for (let api_name in apis) {
            let api = apis[api_name];
            let _headers = _.extend({}, api.headers);
            if (token) {
                // inject Bearer Token
                _headers.authorization = token;
            }
            let api_url = _.template(api.url, Vars_1.default.templateSettings)(params);
            let _request = this.named_request(context, api_name, api_url, _headers);
            _requests.push(_request);
        }
        return new Promise((resolve, reject) => {
            // concurrent "named" requests
            Promise.all(_requests).then((responses => {
                // merge named results
                let results = {};
                for (let r in responses) {
                    _.extend(results, responses[r]);
                }
                resolve(results);
            })).catch(reject);
        });
    }
    static getOperationContext(_op, req, _ctx) {
        let ctx = _.extend({}, _ctx, { op: { resource: _op.resource, actionId: _op.actionId, operationId: _op.operationId } });
        return APIs.getRequestContext(req, ctx);
    }
    static getRequestContext(req, _ctx) {
        let ctx = _.extend({ params: {}, jwt: {}, query: {} }, _ctx, { req: { path: req.path, method: req.method }, params: req.params, query: req.query, jwt: req.jwt || {} });
        return ctx;
    }
}
APIs.DEBUG = false;
exports.default = APIs;
//# sourceMappingURL=APIs.js.map