"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const index_1 = require("../index");
const request = require("request");
const openapi_1 = require("../openapi");
/**
 * plugin to handle JSON body
 *
 * @param context
 * @param options
 * @returns {exports}
 */
class DataDictMiddleware {
    constructor() {
        this.name = "datadict";
    }
    fn(oper, _options) {
        if (_options.url) {
            return this.remote_openapi(oper, _options);
        }
        else {
            return this.local_openapi(oper, _options);
        }
    }
    local_openapi(oper, _options) {
        let context = oper.context;
        let plugin = context.plugins.get("openapi");
        // let options = _.extend({}, this.options, _options);
        let registry = plugin.openapi.schemas;
        let api_url = _options.url;
        context.log({ code: "api:datadict:local", url: api_url });
        return ((req, res) => {
            let params = _.extend({ id: true, ref: true }, req.params, req.query, _options.params);
            let schemas = registry.all(params);
            res.status(200);
            res.send(schemas);
        });
    }
    remote_openapi(oper, _options) {
        let context = oper.context;
        // let options = _.extend({}, this.options, _options);
        let api_url = _options.url;
        context.log({ code: "api:datadict:remote", url: api_url });
        return ((req, res) => {
            let params = _.extend({ id: true, ref: true }, req.params, req.query, _options.params);
            context.log({ code: "api:datadict:request", url: api_url });
            request(api_url, (err, reply) => {
                if (err)
                    return index_1.Utils.sendError(oper.context, res, { code: "api:datadict:remote:schemas", message: err.message, error: err });
                let body_spec = JSON.parse(reply.body);
                let openapi = new openapi_1.OpenAPI(context, body_spec);
                let registry = openapi.schemas;
                let schemas = registry.all(params);
                res.status(200);
                res.send(schemas);
            });
        });
    }
}
exports.default = DataDictMiddleware;
//# sourceMappingURL=DataDict.js.map