"use strict";
/*************************************************************************
 *
 * Troven CONFIDENTIAL
 * __________________
 *
 *  (c) 2017-2020 Troven Ventures Pty Ltd
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Troven Pty Ltd and its licensors,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Troven Pty Ltd
 * and its suppliers and may be covered by International and Regional Patents,
 * patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Troven Pty Ltd.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const jwt = require("jsonwebtoken");
const index_1 = require("../index");
/*

curl -v --data "grant_type=password&client_id=test-lab&client_secret=abcdefgg-0a50-4929-967a-178bf454314a&username=test-lab&password=$OAUTH_PASS" https://oauth.pro/auth/realms/example/protocol/openid-connect/token

curl -v https://oauth.pro/auth/realms/example/protocol/openid-connect/certs

 */
/**
 * Authenticate JWT
 * ----------------
 * Ensures a JWT is correctly formed and signed by the trusted KeycloakIdP.
 *
 * Loads the server public certificate from file or URL.
 *
 * Can enforce mandatory claims for all resources.
 *
 * @type {{name: string, title: string, description: string, usecase: [string], _loadPublicKey: module.exports._loadPublicKey, fn: module.exports.fn}}
 */
class jwt_authenticate {
    constructor(plugin) {
        this.plugin = plugin;
        this.name = "api.jwt_authenticate";
        this.title = "ensure a JWT is authentic (correctly formed and signed)";
    }
    fn(operation, _options) {
        let context = operation.context;
        return function (req, res, next) {
            if (req.jwt) {
                return next(); // no need to run again
            }
            let authenticated = this.authenticate(context, operation, req, res, this.plugin.secretOrCertificate);
            if (!authenticated) {
                return index_1.Utils.sendError(context, res, { statusCode: 401, code: "api:jwt:not-authenticated", message: "missing or invalid token" });
            }
            next();
        };
    }
    static authenticate(context, op, req, res, secretOrCertificate) {
        let req_uuid = res.getHeader("x-request-id");
        let token = index_1.JWT.getBearerTokenOrSendError(context, req);
        if (!token) {
            context.log({ code: "api:jwt:missing", message: "missing token", resource: op.resource, request_id: req_uuid });
            return false;
        }
        // validate and decode
        if (secretOrCertificate) {
            req.jwt = jwt.verify(token, secretOrCertificate);
        }
        else {
            req.jwt = jwt.decode(token);
            let log = { code: "api:jwt:decoded", message: "NOT VERIFIED", resource: op.resource, request_id: req_uuid };
            context.warn(log);
            context.audit(log);
        }
        return req.jwt ? true : false;
    }
}
exports.default = jwt_authenticate;
//# sourceMappingURL=jwt_authenticate.js.map