"use strict";
/*************************************************************************
 *
 * Troven CONFIDENTIAL
 * __________________
 *
 *  (c) 2017-2020 Troven Ventures Pty Ltd
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Troven Pty Ltd and its licensors,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Troven Pty Ltd
 * and its suppliers and may be covered by International and Regional Patents,
 * patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Troven Pty Ltd.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const index_1 = require("../index");
const JWT_1 = require("../helpers/JWT");
//import * as request_uuid from "./request_uuid";
/**
 * Authorize JWT
 * -------------
 * Ensure the JWT contains specified claims.
 *
 * Unless disabled, an audit event is emitted.
 *
 */
class jwt_authorize {
    constructor(plugin) {
        this.plugin = plugin;
        this.name = "api.jwt_authorize";
        this.title = "ensure a JWT contains specified claims";
    }
    fn(operation, options) {
        let context = operation.context;
        return function (req, res, next) {
            let req_uuid = res.getHeader("x-request-id");
            let jwt = req.jwt;
            if (!jwt) {
                return index_1.Utils.sendError(context, res, { statusCode: 401, code: "api:jwt:token:missing", message: "missing token", request_id: req_uuid });
            }
            let now = Date.now() / 1000; // now in seconds;
            // check JWT: has not expired
            if (now > jwt.exp) {
                return index_1.Utils.sendError(this.context, res, { statusCode: 403, code: "api:jwt:expired", message: "expired", request_id: req_uuid, resource: operation.resource, method: operation.actionId, expires: new Date(jwt.exp * 1000), jwt: jwt });
            }
            // check JWT: not yet issued
            if (now < jwt.nbf) {
                return index_1.Utils.sendError(this.context, res, { statusCode: 403, code: "api:jwt:nbf", message: "not active", request_id: req_uuid, resource: operation.resource, method: operation.actionId, nbf: new Date(jwt.nbf * 1000), jwt: jwt });
            }
            // check for mandatory claims in our JWT
            let access_granted = JWT_1.default.hasClaims(jwt, options.claims, context);
            if (!access_granted) {
                return index_1.Utils.sendError(context, res, { statusCode: 403, code: "api:jwt:claims", message: "invalid claims", request_id: req_uuid, resource: operation.resource, method: operation.actionId, jwt: jwt, claims: options.claims });
            }
            context.audit({ uuid: context.uuid, jti: jwt.jti, claims: options.claims, request_id: req_uuid, code: "api:jwt:ok", message: "authorized" });
            // we're good - continue processing request ...
            next();
        };
    }
}
exports.default = jwt_authorize;
//# sourceMappingURL=jwt_authorize.js.map