"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const middleware_1 = require("../middleware");
const fs = require("fs");
const axios_1 = require("axios");
// import keycloak_public from 'get-keycloak-public-key'
/**
 * plugin has injects middleware 'after' every response is generated
 *
 * @param context
 * @param options
 * @returns {exports}
 */
class JWTPlugin {
    constructor() {
        this.ALLOWED_ALGOS = ['RS256'];
        this.name = "jwt";
        this.title = "Inject plugins 'jwt' middleware";
        this.secretOrCertificate = null;
        this._loadPublicKey = function (certificateFile) {
            return fs.readFileSync(certificateFile, "UTF-8");
        };
        this._downloadPublicKey = function (url) {
            axios_1.default({ url: url }).then(r => {
                this.secretOrCertificate = "-----BEGIN RSA PUBLIC KEY-----\n" + r.data.public_key + "\n-----END RSA PUBLIC KEY-----";
                this.context.log({ code: "plugin:jwt:cert:loaded", secretOrCertificate: this.secretOrCertificate });
            }).catch(err => {
                this.context.fatal({ code: "plugin:jwt:cert:load:failed", message: err.message, url: url });
            });
        };
    }
    install(_context, _options) {
        this.context = _context;
        let options = _options;
        this.secretOrCertificate = options.secret || null;
        if (options.certificateFile) {
            this.secretOrCertificate = this._loadPublicKey(options.certificateFile);
        }
        if (options.realmURL) {
            this._downloadPublicKey(options.realmURL);
        }
        this.algo = options.algo || this.ALLOWED_ALGOS[0];
        _context.middleware.set(new middleware_1.jwt_authenticate(this));
        _context.middleware.set(new middleware_1.jwt_authorize(this));
        this.context.log({ code: "plugin:jwt", algo: this.algo, verify: options.realmURL ? true : this.secretOrCertificate ? true : false });
    }
}
exports.default = JWTPlugin;
//# sourceMappingURL=jwt.js.map