"use strict";
/*************************************************************************
 *
 * Troven CONFIDENTIAL
 * __________________
 *
 *  (c) 2017-2020 Troven Ventures Pty Ltd
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Troven Pty Ltd and its licensors,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Troven Pty Ltd
 * and its suppliers and may be covered by International and Regional Patents,
 * patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Troven Pty Ltd.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const openapi_1 = require("../openapi/");
const jwt_openapi_1 = require("../middleware/jwt_openapi");
const _ = require("lodash");
const assert = require("assert");
const express_1 = require("express");
const middleware_1 = require("../middleware");
// import { Vars } from "../helpers";
/**
 * openapi
 * ---------
 * Used when a method is not found
 *
 * @type {{name: string, title: string, fn: module.exports.fn}}
 */
class openapi {
    constructor() {
        this.name = "openapi";
        this.title = "OpenAPI plugin";
        this.openapi = null;
    }
    install(context, options) {
        this.context = context;
        // merge spec from config.openapi and plugins.openapi
        let spec = _.extend({}, context.config.openapi, options);
        this.openapi = new openapi_1.OpenAPI(context, spec);
        // ensure all plugins are installed before declarative routing begins ...
        context.bus.on("api:start", () => {
            this.pipeline = context.plugins.get("pipeline");
            assert(this.pipeline, "pipeline not installed");
            this.jwt = context.plugins.get("jwt");
            context.middleware.set(new middleware_1.jwt_authenticate(this.jwt));
            context.middleware.set(new middleware_1.jwt_authorize(this.jwt));
            context.middleware.set(new jwt_openapi_1.default(this.jwt, this.openapi));
            this.reset();
        });
    }
    reset() {
        // wire-up express routing
        this.api = express_1.Router({ mergeParams: true });
        this.context.api.use(this.api);
        this.route_paths(this.openapi.paths);
    }
    route_paths(paths) {
        _.each(paths.ops, (op) => {
            this.route(op);
        });
        return this.api;
    }
    route(op) {
        assert(this.api, "openapi race condition");
        assert(this.pipeline && this.pipeline.middleware, "can't route before pipeline installed");
        let endpoint = this.api[op.actionId];
        assert(endpoint, "unknown method: " + op.actionId + " for " + op.resource);
        let fn = this.pipeline.middleware.fn(op, {});
        //transform all OpenAPI {{param}} into Express :param
        let mount_path = op.resource.replace(new RegExp("{", 'g'), ":").replace(new RegExp("}", 'g'), "");
        let args = [mount_path];
        if (op.security) {
            op.feature.jwt_openapi = { matchers: {} };
            let jwt_openapi = this.pipeline.middleware.feature("jwt_openapi");
            assert(jwt_openapi, "missing 'jwt_openapi' middleware");
            let security = jwt_openapi.fn(op, { security: op.security });
            args.push(security);
        }
        this.openapi.paths.add(op);
        // attach fn() to router
        args.push(fn);
        endpoint.apply(this.api, args);
        this.context.log({ code: "api:plugin:openapi", actionId: op.actionId, resource: op.resource, route: mount_path, featureId: op.featureId, operationId: op.operationId, security: op.security ? true : false });
        return endpoint;
    }
}
exports.default = openapi;
//# sourceMappingURL=openapi.js.map