
import * as request from "request";
import { IChassisContext, IOperation } from "../interfaces";
import Vars from "./Vars";
import * as _ from "lodash";
import * as http from "http";
import * as https from "https";
import { URL } from "url";

export default class APIs {
    static DEBUG = false

    constructor() {
    }

    static get( _context: IChassisContext, _url: string ): Promise<any> {
        let url = new URL(_url);
        let client = url.protocol=="https:"?https:http;
        return new Promise(( resolve, reject ) => {
            try {
                client.get(url, (res) => {
                    resolve(res);
                })
            } catch(err) {
                console.log("OOPS: %s", err);
                reject(err);
            }
        });
    }

    static request( context: IChassisContext, url: string, _headers: any ): Promise<any> {

        return new Promise(( resolve, reject ) => {
            // call backend API
            APIs.DEBUG && context.log({ code: "feature:helpers:request:api", url: url, headers: _headers });

            request({ uri: url, headers: _headers }, function (err: any, httpResponse: any, body: any) {
                if (err || !httpResponse) {
                    context.error({ code: "feature:helpers:api:failed", url: url, httpResponse: httpResponse?true:false, error: err?err.toString():"no error" });
                    reject(err);
                } else {
                    APIs.DEBUG && context.log({ code: "feature:helpers:api:ok", url: url, body: body });
                    try {
                        resolve( JSON.parse(body) );
                    } catch (e) {
                        resolve(body);
                    }
                }
            });
        });
    }

    static named_request( context: IChassisContext, api_name: string, url: string, _headers: any ): Promise<any> {

        return new Promise(( resolve, reject ) => {
            // call backend API
            APIs.DEBUG && context.log({ code: "feature:render:api", api: api_name, url: url, headers: _headers });

            request({ uri: url, headers: _headers }, function (err: any, httpResponse: any, body: any) {
                if (err || !httpResponse) {
                    context.error({ code: "feature:render:api:failed", api: api_name, url: url, httpResponse: httpResponse?true:false, error: err?err.toString():"no error" });
                    reject(err);
                } else {
                    try {
                        let result = {};
                        result[api_name] = JSON.parse(body);
                        resolve( result );
                    } catch (e) {
                        reject(e);
                    }
                }
            });
        });
    }

    /**
     * Collect a set of API responses as promise
     * 
     * @param apis 
     * @param params 
     * @param token 
     */

    static requests(context: IChassisContext, apis: APIRequests, params: any, token?: string): Promise<APIResponses> {
        let _requests = [];

        // iterate over set of APIs
        for (let api_name in apis) {
            let api = apis[api_name];

            let _headers: any = _.extend({}, api.headers);
            if (token) {
                // inject Bearer Token
                _headers.authorization = token;
            }
            let api_url = _.template(api.url, Vars.templateSettings)(params);
            let _request = this.named_request( context, api_name, api_url, _headers);
            _requests.push( _request );
        }

        return new Promise( (resolve, reject) => {

            // concurrent "named" requests
            Promise.all(_requests).then( (responses => {
                // merge named results
                let results: APIResponses = {};
                for(let r in responses) {
                    _.extend(results, responses[r]);
                }
                resolve(results);
            })).catch( reject );

        });
    }

    public static getOperationContext( _op: IOperation, req: any, _ctx?: any): APIContext {
        let ctx = _.extend({}, _ctx, { op: { resource: _op.resource , actionId: _op.actionId , operationId: _op.operationId } } );
        return APIs.getRequestContext(req, ctx)
    }

    public static getRequestContext(req: any, _ctx?: any): APIContext {
        let ctx = _.extend({ params: {}, jwt: {}, query: {} }, _ctx,
            { req: { path: req.path, method: req.method }, params: req.params, query: req.query, jwt: req.jwt || {} } );
        return ctx as APIContext;
    }
} 

export interface APIRequests {
    [ name: string]: APIRequest
}

export interface APIRequest {
    url: string;
    headers?: any;
}

export interface APIResponses {
    [ name: string]: any;
}

export interface APIContext {
    req: any
    op: any
    jwt: any
    params: any
    query: any
}