/*************************************************************************
 *
 * Troven CONFIDENTIAL
 * __________________
 *
 *  (c) 2017-2020 Troven Ventures Pty Ltd
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Troven Pty Ltd and its licensors,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Troven Pty Ltd
 * and its suppliers and may be covered by International and Regional Patents,
 * patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Troven Pty Ltd.
 */

import * as _ from "lodash";
import {IOperation, IChassisContext} from "../index";
import { Handler, Request, Response, NextFunction } from "express-serve-static-core";
import { Vars } from "../helpers";
import graphql_plugin  from "../plugins/graphql";
import proxy from "./proxy"

/**
 * GraphQL proxy middleware
 * --------------
 * Proxy/Transform request into GraphQL
 *
 * @type {{name: string, title: string, when: [string,string], fn: module.exports.fn, Decrypter: module.exports.Decrypter}}
 */
export default class graphql extends proxy {

    name = "api.graphql";
    title = "Transform REST API request into GraphQL";

    constructor(protected plugin: graphql_plugin) {
        super();
    }

    fn(oper: IOperation, _options: any): Handler {
        let context: IChassisContext = oper.context;

        let options = _.extend({ proxy: {}, query: {}, unwrap: "" }, _options, oper.feature);

        options.proxy = _.defaults(options.proxy, this.plugin.options.proxy);

        context.log({ code: "api:graphql", message: oper.resource+" -> "+_options.target, proxyPath: oper.resource, options: options });

        // TODO: unwrap response envelope
        // if (options.unwrap) {
        //     options.selfHandleResponse = true;
        //     options.proxy.onProxyRes = (proxyRes , _req, _res) => {
        //         context.log({ code: "api:graphql:unwrap", message: oper.resource+" -> "+_options.target, proxyPath: oper.resource, res: proxyRes.body });
        //         var body = [];
        //         proxyRes.on('data', function (chunk) {
        //             body.push(chunk);
        //         });
        //         proxyRes.on('end', function () {
        //             body = Buffer.concat(body).toString();
        //             console.log("res from proxied server:", body);
        //             res.end("my response to cli");
        //         });
        //     }
        // }

        let proxy_forwarder = super.fn(oper, options.proxy);

        return (req: Request, res: Response, _next: NextFunction) => {

            // interpolate GraphQL query that supports req.params & req.query params, and (optional) JWT
            let jwt = (req as any).jwt || {};
            let vars = _.extend({ resource: oper.resource, params: req.params, query: req.query, jwt: jwt });
            let query$ = options.query.toString();
            let query = Vars.$(query$, vars);

            // update request with re-written GraphQL query
            req.body = { query: query, variables: null };

            // consistent API request
            req.method = "POST";
            req.headers["content-type"] = "application/json";

            if (options.debug) {
                context.log({ code: "api:graphql", message: oper.resource+" -> "+_options.target, proxyPath: oper.resource, query: req.body });
            }

            // send to GraphQL endpoint via proxy
            proxy_forwarder(req, res, _next);
        }
    }
}