

import {IChassisContext, IChassisPlugin } from "../index";
import { jwt_authenticate, jwt_authorize } from "../middleware";
import * as _ from "lodash";
import * as fs from "fs";
import axios from 'axios'
// import keycloak_public from 'get-keycloak-public-key'

/**
 * plugin has injects middleware 'after' every response is generated
 *
 * @param context
 * @param options
 * @returns {exports}
 */
export default class JWTPlugin implements IChassisPlugin {
    ALLOWED_ALGOS = ['RS256']

    name = "jwt";
    title = "Inject plugins 'jwt' middleware";
    context: IChassisContext;
    secretOrCertificate: string = null;
    algo:string

    install(_context: IChassisContext, _options: any) {
        this.context = _context;
        let options = _options as any;

        this.secretOrCertificate = options.secret || null;
        if (options.certificateFile) {
            this.secretOrCertificate = this._loadPublicKey(options.certificateFile);
        }
        if (options.realmURL) {
            this._downloadPublicKey(options.realmURL);
        }
        this.algo = options.algo || this.ALLOWED_ALGOS[0]

        _context.middleware.set( new jwt_authenticate(this) );
        _context.middleware.set( new jwt_authorize(this) );

        this.context.log({ code: "plugin:jwt", algo: this.algo, verify: options.realmURL?true:this.secretOrCertificate?true:false });
    }

    _loadPublicKey =  function(certificateFile) {
        return fs.readFileSync(certificateFile, "UTF-8");
    };

    _downloadPublicKey =  function(url) {
        axios( {url:url }).then(r => {
            this.secretOrCertificate = "-----BEGIN RSA PUBLIC KEY-----\n"+r.data.public_key+"\n-----END RSA PUBLIC KEY-----";
            this.context.log({ code: "plugin:jwt:cert:loaded", secretOrCertificate: this.secretOrCertificate });
        }).catch( err => {
            this.context.fatal( { code: "plugin:jwt:cert:load:failed", message: err.message, url: url })
        })
    };

}